@@ -1,7 +1,7 @@
 package Sort::BySpec;
 
-our $DATE = '2015-12-29'; # DATE
-our $VERSION = '0.02'; # VERSION
+our $DATE = '2017-02-17'; # DATE
+our $VERSION = '0.03'; # VERSION
 
 use 5.010001;
 use strict 'subs', 'vars';
@@ -41,12 +41,17 @@ This is just like `xform` in `Sort::ByExample`.
 
 _
         },
+        reverse => {
+            summary => 'If set to true, will reverse the sort order',
+            schema => ['bool*', is=>1],
+        },
         array => {
             schema => 'array*',
         },
     },
     result => {
         summary => 'Sorted array, or sort coderef',
+        schema => ['any*', of=>['array*','code*']],
         description => <<'_',
 
 If array is specified, will returned the sorted array. If array is not specified
@@ -64,6 +69,14 @@ _
                 array => [1, 2, 3, 'bar', 'a', 'b', 'c', 'baz'],
             },
         },
+        {
+            summary => 'Like previous example, but reversed',
+            args => {
+                spec => ['foo', 'bar', 'baz'],
+                array => [1, 2, 3, 'bar', 'a', 'b', 'c', 'baz'],
+                reverse => 1,
+            },
+        },
         {
             summary => 'Put integers first (in descending order), then '.
                 'a sequence of scalars, then others (in ascending order)',
@@ -124,34 +137,37 @@ sub sort_by_spec {
         return($j+1);
     };
 
-    my $cmp = sub {
-        my ($a, $b);
-
-        if (@_ >= 2) {
-            $a = $_[0];
-            $b = $_[1];
-        } else {
-            my $caller = caller();
-            $a = ${"caller\::a"};
-            $b = ${"caller\::b"};
-        }
-
-        if ($xform) {
-            $a = $xform->($a);
-            $b = $xform->($b);
-        }
-
-        my ($rank_a, $sortsub) = $code_get_rank->($a);
-        my ($rank_b          ) = $code_get_rank->($b);
-
-        if ($rank_a != $rank_b) {
-            return $rank_a <=> $rank_b;
-        }
-        return 0 unless $sortsub;
-        return $sortsub->($a, $b);
-    };
-
     if ($args{_return_cmp}) {
+        my $cmp = sub {
+            my ($a, $b);
+
+            if (@_ >= 2) {
+                $a = $_[0];
+                $b = $_[1];
+            } else {
+                my $caller = caller();
+                $a = ${"caller\::a"};
+                $b = ${"caller\::b"};
+            }
+
+            if ($xform) {
+                $a = $xform->($a);
+                $b = $xform->($b);
+            }
+
+            if ($args{reverse}) {
+                ($a, $b) = ($b, $a);
+            }
+
+            my ($rank_a, $sortsub) = $code_get_rank->($a);
+            my ($rank_b          ) = $code_get_rank->($b);
+
+            if ($rank_a != $rank_b) {
+                return $rank_a <=> $rank_b;
+            }
+            return 0 unless $sortsub;
+            return $sortsub->($a, $b);
+        };
         return $cmp;
     } else {
         # use schwartzian transform to speed sorting longer lists
@@ -179,7 +195,9 @@ $SPEC{cmp_by_spec} = do {
     $meta->{summary} = 'Create a compare subroutine to be used in sort()';
     $meta->{args} = { %{$meta->{args}} };
     delete $meta->{args}{array};
-    delete $meta->{result};
+    $meta->{result} = {
+        schema => ['code*'],
+    };
     delete $meta->{examples};
     $meta;
 };
@@ -205,7 +223,7 @@ Sort::BySpec - Sort array (or create a list sorter) according to specification
 
 =head1 VERSION
 
-This document describes version 0.02 of Sort::BySpec (from Perl distribution Sort-BySpec), released on 2015-12-29.
+This document describes version 0.03 of Sort::BySpec (from Perl distribution Sort-BySpec), released on 2017-02-17.
 
 =head1 SYNOPSIS
 
@@ -289,7 +307,11 @@ for more complex matching:
 =head1 FUNCTIONS
 
 
-=head2 cmp_by_spec(%args) -> any
+=head2 cmp_by_spec
+
+Usage:
+
+ cmp_by_spec(%args) -> code
 
 Create a compare subroutine to be used in sort().
 
@@ -299,6 +321,10 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
+=item * B<reverse> => I<bool>
+
+If set to true, will reverse the sort order.
+
 =item * B<spec>* => I<array>
 
 =item * B<xform> => I<code>
@@ -309,10 +335,14 @@ This is just like C<xform> in C<Sort::ByExample>.
 
 =back
 
-Return value:  (any)
+Return value:  (code)
 
 
-=head2 sort_by_spec(%args) -> any
+=head2 sort_by_spec
+
+Usage:
+
+ sort_by_spec(%args) -> array|code
 
 Sort array (or create a list sorter) according to specification.
 
@@ -323,8 +353,20 @@ Examples:
 =item * Sort according to a sequence of scalars (like Sort::ByExample):
 
  sort_by_spec(
-   array => [1, 2, 3, "bar", "a", "b", "c", "baz"],
-   spec  => ["foo", "bar", "baz"]
+ spec  => ["foo", "bar", "baz"],
+   array => [1, 2, 3, "bar", "a", "b", "c", "baz"]
+ );
+
+Result:
+
+ ["bar", "baz", 1, 2, 3, "a", "b", "c"]
+
+=item * Like previous example, but reversed:
+
+ sort_by_spec(
+ spec    => ["foo", "bar", "baz"],
+   array   => [1, 2, 3, "bar", "a", "b", "c", "baz"],
+   reverse => 1
  );
 
 Result:
@@ -334,8 +376,16 @@ Result:
 =item * Put integers first (in descending order), then a sequence of scalars, then others (in ascending order):
 
  sort_by_spec(
-   array => ["qux", "b", "a", "bar", "foo", 1, 10, 2],
-   spec  => [qr/\A\d+\z/, sub { ... }, "foo", "bar", "baz", qr//, sub { ... }]
+ spec  => [
+              qr/\A\d+\z/,
+              sub { $_[1] <=> $_[0] },
+              "foo",
+              "bar",
+              "baz",
+              qr//,
+              sub { $_[0] cmp $_[1] },
+            ],
+   array => ["qux", "b", "a", "bar", "foo", 1, 10, 2]
  );
 
 Result:
@@ -352,6 +402,10 @@ Arguments ('*' denotes required arguments):
 
 =item * B<array> => I<array>
 
+=item * B<reverse> => I<bool>
+
+If set to true, will reverse the sort order.
+
 =item * B<spec>* => I<array>
 
 =item * B<xform> => I<code>
@@ -362,26 +416,20 @@ This is just like C<xform> in C<Sort::ByExample>.
 
 =back
 
-Return value: Sorted array, or sort coderef (any)
+Return value: Sorted array, or sort coderef (array|code)
 
 
 If array is specified, will returned the sorted array. If array is not specified
 in the argument, will return a sort subroutine that can be used to sort a list
 and return the sorted list.
 
-=head1 SEE ALSO
-
-L<Sort::ByExample>
-
-L<Bencher::Scenario::SortBySpec>
-
 =head1 HOMEPAGE
 
 Please visit the project's homepage at L<https://metacpan.org/release/Sort-BySpec>.
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/perlancar/perl-Sort-ByExample-Clusters>.
+Source repository is at L<https://github.com/perlancar/perl-Sort-BySpec>.
 
 =head1 BUGS
 
@@ -391,13 +439,19 @@ When submitting a bug or request, please include a test-file or a
 patch to an existing test-file that illustrates the bug or desired
 feature.
 
+=head1 SEE ALSO
+
+L<Sort::ByExample>
+
+L<Bencher::Scenario::SortBySpec>
+
 =head1 AUTHOR
 
 perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2015 by perlancar@cpan.org.
+This software is copyright (c) 2017, 2015 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
